package com.ocpsoft.pretty;

/*
 * PrettyFaces is an OpenSource JSF library to create bookmarkable URLs.
 * Copyright (C) 2009 - Lincoln Baxter, III <lincoln@ocpsoft.com> This program
 * is free software: you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version. This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see the file COPYING.LESSER
 * or visit the GNU website at <http://www.gnu.org/licenses/>.
 */
import java.io.IOException;
import java.util.List;

import javax.faces.application.ProjectStage;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.SAXException;

import com.ocpsoft.pretty.config.PrettyConfig;
import com.ocpsoft.pretty.config.PrettyConfigurator;
import com.ocpsoft.pretty.config.faces.WebXmlParser;
import com.ocpsoft.pretty.config.mapping.PathParameter;
import com.ocpsoft.pretty.config.rewrite.Redirect;
import com.ocpsoft.pretty.config.rewrite.RewriteRule;
import com.ocpsoft.pretty.faces.DynaviewEngine;
import com.ocpsoft.pretty.rewrite.RewriteEngine;
import com.ocpsoft.pretty.servlet.PrettyFacesWrappedRequest;
import com.ocpsoft.pretty.servlet.PrettyFacesWrappedResponse;
import com.ocpsoft.pretty.util.FacesFactory;
import com.ocpsoft.pretty.util.QueryString;

/**
 * @author Lincoln Baxter, III <lincoln@ocpsoft.com>
 */
public class PrettyFilter implements Filter
{
    private static final Log log = LogFactory.getLog(PrettyFilter.class);

    public static final String CONFIG_FILES_ATTR = "com.ocpsoft.pretty.CONFIG_FILES";
    private static final String REWRITE_OCCURRED_KEY = "com.ocpsoft.pretty.rewrite";

    private final WebXmlParser webXmlParser = new WebXmlParser();
    private final DynaviewEngine dynaview = new DynaviewEngine();
    private final RewriteEngine rewriteEngine = new RewriteEngine();

    private ServletContext servletContext;
    private String facesDynaViewId = "";

    /**
     * Determine if the current request is mapped using PrettyFaces. If it is,
     * process the pattern, storing parameters into the request map, then
     * forward the request to the specified viewId.
     */
    public void doFilter(final ServletRequest req, final ServletResponse resp, final FilterChain chain)
            throws IOException, ServletException
    {
        ServletResponse response = new PrettyFacesWrappedResponse((HttpServletResponse) resp, getConfig());

        try
        {
            if (!ProjectStage.Production.equals(FacesFactory.getApplication().getProjectStage()))
            {
                loadConfiguration();
            }

            // TODO refactor this into a chain?
            boolean canonicalizationOccurred = rewrite((HttpServletRequest) req, (HttpServletResponse) resp);
            if (!canonicalizationOccurred)
            {
                String requestURI = getRequestURI(req);
                if (getConfig().isURLMapped(requestURI))
                {
                    PrettyContext prettyContext = PrettyContext.newInstance((HttpServletRequest) req);

                    String viewId = prettyContext.getCurrentViewId();
                    if (!response.isCommitted())
                    {
                        if (prettyContext.shouldProcessDynaview())
                        {
                            log.trace("Forwarding mapped request [" + requestURI + "] to dynaviewId [" + viewId + "]");
                            req.getRequestDispatcher(facesDynaViewId).forward(req, response);
                        }
                        else
                        {
                            List<PathParameter> params = prettyContext.getCurrentMapping().getPatternParser().parse(
                                    requestURI);
                            QueryString query = QueryString.build(params);

                            ServletRequest request = new PrettyFacesWrappedRequest((HttpServletRequest) req, query
                                    .getParameterMap());

                            log.trace("Forwarding mapped request [" + requestURI + "] to JSF viewId [" + viewId + "]");
                            req.getRequestDispatcher(viewId).forward(request, response);
                        }
                    }
                }
                else
                {
                    ensurePopulatedContext(req);
                    log.trace("Request is not mapped using PrettyFaces. Continue.");
                    chain.doFilter(req, response);
                }
            }
        }
        catch (Exception e)
        {
            throw new ServletException(e);
        }
    }

    /**
     * Apply the given list of {@link RewriteRule}s to the URL (in order,)
     * perform a
     * redirect/forward if required. Canonicalization is only invoked if it has
     * not previously been invoked on this request. This method operates on the
     * requestUri, excluding contextPath.
     * 
     * @return True if forward/redirect occurred, false if not.
     */
    private boolean rewrite(final HttpServletRequest req, final HttpServletResponse resp)
    {
        /*
         * FIXME Refactor this horrible method.
         */
        boolean redirected = false;
        if (!Boolean.TRUE.equals(req.getAttribute(REWRITE_OCCURRED_KEY)))
        {
            String url = getRequestURI(req);

            try
            {
                String temp = url;
                for (RewriteRule c : getConfig().getGlobalRewriteRules())
                {
                    if (c.matches(temp))
                    {
                        temp = rewriteEngine.processInbound(c, temp);
                        if (!Redirect.CHAIN.equals(c.getRedirect()))
                        {
                            /*
                             * An HTTP redirect has been triggered; issue one if
                             * we have a url or if the current url has been
                             * modified.
                             */
                            if (c.getUrl().isEmpty() && !url.equals(temp))
                            {
                                String queryString = req.getQueryString() == null ? "" : "?" + req.getQueryString();
                                resp.setHeader("Location", resp.encodeRedirectURL(req.getContextPath() + temp
                                        + queryString));
                                resp.sendError(c.getRedirect().getStatus());
                                redirected = true;
                                break;
                            }
                            else if (!c.getUrl().isEmpty())
                            {
                                resp.setHeader("Location", resp.encodeRedirectURL(temp));
                                resp.sendError(c.getRedirect().getStatus());
                                redirected = true;
                                break;
                            }
                        }
                    }
                }

                if (!url.equals(temp) && redirected == false)
                {
                    /*
                     * The URL was modified, but no redirect occurred; forward
                     * instead.
                     */
                    req.setAttribute(REWRITE_OCCURRED_KEY, true);
                    req.getRequestDispatcher(temp).forward(req, resp);
                    redirected = true;
                }

            }
            catch (Exception e)
            {
                throw new PrettyException("Error occurred during canonicalization of request <[" + url + "]>", e);
            }
            finally
            {
                req.setAttribute(REWRITE_OCCURRED_KEY, true);
            }
        }
        return redirected;
    }

    private void ensurePopulatedContext(final ServletRequest request)
    {
        PrettyContext.getCurrentInstance((HttpServletRequest) request);
    }

    public PrettyConfig getConfig()
    {
        if (servletContext == null)
        {
            log.warn("PrettyFilter is not registered in web.xml, but is registered with JSF "
                    + "Navigation and Action handlers -- this could cause unpredictable behavior.");
            return new PrettyConfig();
        }
        return (PrettyConfig) servletContext.getAttribute(CONFIG_FILES_ATTR);
    }

    private void loadConfiguration() throws ServletException
    {
        try
        {
            final PrettyConfigurator configurator = new PrettyConfigurator(servletContext);
            webXmlParser.parse(servletContext);
            facesDynaViewId = dynaview.buildDynaViewId(webXmlParser.getFacesMapping());
            log.trace("Setting config into ServletContext");
            servletContext.setAttribute(CONFIG_FILES_ATTR, configurator.configure());
        }
        catch (IOException e)
        {
            throw new ServletException("Error configuring Pretty Faces, could not load the configuration.", e);
        }
        catch (SAXException e)
        {
            throw new ServletException("Error configuring Pretty Faces, could not parse the configuration.", e);
        }
    }

    private String getRequestURI(final ServletRequest request)
    {
        String requestURI = ((HttpServletRequest) request).getRequestURI();
        String contextPath = ((HttpServletRequest) request).getContextPath();
        if (requestURI.startsWith(contextPath))
        {
            requestURI = requestURI.substring(contextPath.length());
        }
        return requestURI;
    }

    /**
     * Load and cache configurations
     */
    public void init(final FilterConfig filterConfig) throws ServletException
    {
        log.info("PrettyFilter starting up...");
        servletContext = filterConfig.getServletContext();
        loadConfiguration();
        log.info("PrettyFilter initialized.");
    }

    public void destroy()
    {}
}