/*
 * PrettyFaces is an OpenSource JSF library to create bookmarkable URLs.
 * Copyright (C) 2009 - Lincoln Baxter, III <lincoln@ocpsoft.com> This program
 * is free software: you can redistribute it and/or modify it under the terms of
 * the GNU Lesser General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version. This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details. You should have received a copy of the GNU Lesser General
 * Public License along with this program. If not, see the file COPYING.LESSER
 * or visit the GNU website at <http://www.gnu.org/licenses/>.
 */
package com.ocpsoft.pretty.faces.component;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.component.UIParameter;

import com.ocpsoft.pretty.PrettyException;
import com.ocpsoft.pretty.config.mapping.UrlMapping;
import com.ocpsoft.pretty.config.mapping.QueryParameter;
import com.ocpsoft.pretty.util.QueryString;
import com.ocpsoft.pretty.util.URLPatternParser;

/**
 * @author Lincoln Baxter, III <lincoln@ocpsoft.com>
 */
public class PrettyURLBuilder
{
    /**
     * Extract any {@link UIParameter} objects from a given component. These
     * parameters are what PrettyFaces uses to communicate with the JSF
     * component tree
     * 
     * @param component
     * @return A list of {@link UIParameter} objects, or an empty list if no
     *         {@link UIParameter}s were contained within the component
     *         children.
     */
    public List<UIParameter> extractParameters(final UIComponent component)
    {
        List<UIParameter> results = new ArrayList<UIParameter>();
        for (UIComponent child : component.getChildren())
        {
            if (child instanceof UIParameter)
            {
                results.add((UIParameter) child);
            }
        }
        return results;
    }

    public String buildMappedUrl(final UrlMapping urlMapping, final List<UIParameter> parameters)
    {
        if (urlMapping != null)
        {
            URLPatternParser parser = new URLPatternParser(urlMapping.getPattern());
            List<String> pathParams = new ArrayList<String>();

            if (parameters.size() == 1
                    && (parameters.get(0).getValue() instanceof List<?> || parameters.get(0).getValue() == null))
            {
                return parser.getMappedUrl(parameters.get(0).getValue());
            }

            List<QueryParameter> queryParams = new ArrayList<QueryParameter>();
            for (UIParameter parameter : parameters)
            {
                try
                {
                    String name = parameter.getName();
                    Object value = parameter.getValue();

                    if (name == null && value != null)
                    {
                        pathParams.add(URLEncoder.encode(value.toString(), "UTF-8"));
                    }
                    else
                    {
                        List<?> values = null;
                        if (value != null && value.getClass().isArray())
                        {
                            values = Arrays.asList((Object[]) value);
                        }
                        else if (value instanceof List<?>)
                        {
                            values = (List<?>) value;
                        }
                        else if (value != null)
                        {
                            values = Arrays.asList(value);
                        }

                        if (values != null)
                        {
                            for (Object object : values)
                            {
                                String tempValue = null;
                                if (object != null)
                                {
                                    tempValue = object.toString();
                                }
                                queryParams.add(new QueryParameter(name, tempValue));
                            }
                        }
                        else
                        {
                            queryParams.add(new QueryParameter(name, null));
                        }
                    }
                }
                catch (UnsupportedEncodingException e)
                {
                    throw new PrettyException(e);
                }
            }

            return parser.getMappedUrl(pathParams.toArray()) + QueryString.build(queryParams).toQueryString();
        }
        return "";
    }

}
